local CreateFrame = CreateFrame
local GameTooltip = GameTooltip
local Talented = Talented

local L = AceLibrary("AceLocale-2.2"):new("Talented")

local function CreateTexture(base, layer, path, blend)
	local t = base:CreateTexture(nil, layer)
	if path then t:SetTexture(path) end
	if blend then t:SetBlendMode(blend)	end
	return t
end

local trees = Talented.Pool:new()

local function Layout(frame, width, height)
	local texture_height = height / (256+75)
	local texture_width = width / (256+44)
	
	frame:SetWidth(width)
	frame:SetHeight(height)

	frame.topleft:SetWidth(texture_width * 256)
	frame.topleft:SetHeight(texture_height * 256)
	frame.topright:SetWidth(texture_width * 64)
	frame.topright:SetHeight(texture_height * 256)
	frame.bottomleft:SetWidth(texture_width * 256)
	frame.bottomleft:SetHeight(texture_height * 128)
	frame.bottomright:SetWidth(texture_width * 64)
	frame.bottomright:SetHeight(texture_height * 128)
	frame.name:SetWidth(width)
end

local function NewTalentFrame(parent, width, height)
	local texture_height = height / (256+75)
	local texture_width = width / (256+44)

	local frame = CreateFrame("Frame", nil, parent)
	frame:SetWidth(width)
	frame:SetHeight(height)
	frame:SetPoint("TOPLEFT")
	
	
	local t = CreateTexture(frame, "BACKGROUND")
	t:SetWidth(texture_width * 256)
	t:SetHeight(texture_height * 256)
	t:SetPoint("TOPLEFT")
	frame.topleft = t
	
	t = CreateTexture(frame, "BACKGROUND")
	t:SetWidth(texture_width * 64)
	t:SetHeight(texture_height * 256)
	t:SetPoint("TOPLEFT", frame.topleft, "TOPRIGHT")
	frame.topright = t
	
	t = CreateTexture(frame, "BACKGROUND")
	t:SetWidth(texture_width * 256)
	t:SetHeight(texture_height * 128)
	t:SetPoint("TOPLEFT", frame.topleft, "BOTTOMLEFT")
	frame.bottomleft = t
	
	t = CreateTexture(frame, "BACKGROUND")
	t:SetWidth(texture_width * 64)
	t:SetHeight(texture_height * 128)
	t:SetPoint("TOPLEFT", frame.topleft, "BOTTOMRIGHT")
	frame.bottomright = t
	
	local fs = frame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
	fs:SetPoint("TOP", 0, -4)
	fs:SetJustifyH("CENTER")
	fs:SetWidth(width)
	frame.name = fs
	
--~ 		t = CreateTexture(frame, "BORDER")
--~ 		t:SetWidth(32)
--~ 		t:SetHeight(32)
--~ 		t:SetPoint("TOPLEFT")
--~ 		frame.icon = t

	local overlay = CreateFrame("Frame", nil, frame)
	overlay:SetAllPoints(frame)
	
	frame.overlay = overlay

	local clear = CreateFrame("Button", nil, frame)
	frame.clear = clear
	
	local makeTexture = function (path, blend)
		local t = CreateTexture(clear, nil, path, blend)
		t:SetAllPoints(clear)
		return t
	end

	clear:SetNormalTexture(makeTexture("Interface\\Buttons\\CancelButton-Up"))
	clear:SetPushedTexture(makeTexture("Interface\\Buttons\\CancelButton-Down"))
	clear:SetHighlightTexture(makeTexture("Interface\\Buttons\\CancelButton-Highlight", "ADD"))
	
	clear:SetScript("OnClick", function (self)
			Talented:ClearTalentTab(self:GetParent().tab)
		end)
	clear:SetScript("OnEnter", function (self)
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			GameTooltip:SetText(L["Clear this talent branch"])
		end)
	clear:SetScript("OnLeave", function (self)
			if GameTooltip:IsOwned(self) then
				GameTooltip:Hide()
			end
		end)
	
	clear:SetWidth(32)
	clear:SetHeight(32)
	clear:ClearAllPoints()
	clear:SetPoint("TOPRIGHT", -4, -4)
	
	trees:push(frame)
	return frame
end

function Talented:MakeTalentFrame(parent, width, height)
	local tree = trees:next()
	if tree then
		Layout(tree, width, height)
		tree:SetParent(parent)
	else
		tree = NewTalentFrame(parent, width, height)
	end
	return tree
end
